@@ -20,13 +20,19 @@
 
 #include <string.h>
 #include <errno.h>
-
 #include "utils.h"
 #include "indexing.h"
 
-#ifdef FFMS_USE_UTF8_PATHS
+#ifdef _WIN32
 #	define WIN32_LEAN_AND_MEAN
 #	include <windows.h>
+#ifdef FFMS_USE_UTF8_PATHS
+#	include <io.h>
+#	include <fcntl.h>
+extern "C" {
+#	include "libavutil/avstring.h"
+}
+#endif
 #endif
 
 
@@ -75,6 +81,34 @@ int FFMS_Exception::CopyOut(FFMS_ErrorInfo *ErrorInfo) const {
 	return (_ErrorType << 16) | _SubType;
 }
 
+TrackCompressionContext::TrackCompressionContext(MatroskaFile *MF, TrackInfo *TI, unsigned int Track) {
+	CS = NULL;
+	CompressedPrivateData = NULL;
+	CompressedPrivateDataSize = 0;
+	CompressionMethod = TI->CompMethod;
+
+	if (CompressionMethod == COMP_ZLIB) {
+		char ErrorMessage[512];
+		CS = cs_Create(MF, Track, ErrorMessage, sizeof(ErrorMessage));
+		if (CS == NULL) {
+			std::ostringstream buf;
+			buf << "Can't create MKV track decompressor: " << ErrorMessage;
+			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, buf.str());
+		}
+	} else if (CompressionMethod == COMP_PREPEND) {
+		CompressedPrivateData		= TI->CompMethodPrivate;
+		CompressedPrivateDataSize	= TI->CompMethodPrivateSize;
+	} else {
+		throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ, 
+			"Can't create MKV track decompressor: unknown or unsupported compression method");
+	}
+}
+
+TrackCompressionContext::~TrackCompressionContext() {
+	if (CS)
+		cs_Destroy(CS);
+}
+
 int GetSWSCPUFlags() {
 	int Flags = 0;
 
@@ -95,6 +129,8 @@ int GetSWSCPUFlags() {
 int GetPPCPUFlags() {
 	int Flags = 0;
 
+#ifdef WITH_LIBPOSTPROC
+// not exactly a pretty solution but it'll never get called anyway
 	if (CPUFeatures & FFMS_CPU_CAPS_MMX)
 		Flags |= PP_CPU_CAPS_MMX;
 	if (CPUFeatures & FFMS_CPU_CAPS_MMX2)
@@ -103,6 +139,7 @@ int GetPPCPUFlags() {
 		Flags |= PP_CPU_CAPS_3DNOW;
 	if (CPUFeatures & FFMS_CPU_CAPS_ALTIVEC)
 		Flags |= PP_CPU_CAPS_ALTIVEC;
+#endif // WITH_LIBPOSTPROC
 
 	return Flags;
 }
@@ -126,8 +163,20 @@ FFMS_TrackType HaaliTrackTypeToFFTrackType(int TT) {
 	}
 }
 
-void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, CompressedStream *CS, MatroskaReaderContext &Context) {
-	if (CS) {
+const char *GetLAVCSampleFormatName(SampleFormat s) {
+	switch (s) {
+		case SAMPLE_FMT_U8:		return "8-bit unsigned integer";
+		case SAMPLE_FMT_S16:	return "16-bit signed integer";
+		case SAMPLE_FMT_S32:	return "32-bit signed integer";
+		case SAMPLE_FMT_FLT:	return "Single-precision floating point";
+		case SAMPLE_FMT_DBL:	return "Double-precision floating point";
+		default:				return "Unknown";
+	}
+}
+
+void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, TrackCompressionContext *TCC, MatroskaReaderContext &Context) {
+	if (TCC && TCC->CS) {
+		CompressedStream *CS = TCC->CS;
 		unsigned int DecompressedFrameSize = 0;
 
 		cs_NextFrame(CS, FilePos, FrameSize);
@@ -165,7 +214,23 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, CompressedStream *CS,
 			throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_SEEKING, buf.str());
 		}
 
-		if (Context.BufferSize < FrameSize) {
+		if (TCC && TCC->CompressionMethod == COMP_PREPEND) {
+			unsigned ReqBufsize = FrameSize + TCC->CompressedPrivateDataSize + 16;
+			if (Context.BufferSize < ReqBufsize) {
+				Context.BufferSize = FrameSize + TCC->CompressedPrivateDataSize;
+				Context.Buffer = (uint8_t *)realloc(Context.Buffer, ReqBufsize);
+				if (Context.Buffer == NULL)
+					throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_ALLOCATION_FAILED, "Out of memory");
+			}
+
+			/* // maybe faster? maybe not?
+			for (int i=0; i < TCC->CompressedPrivateDataSize; i++)
+				*(Context.Buffer)++ = ((uint8_t *)TCC->CompressedPrivateData)[i];
+			*/
+			// screw it, memcpy and fuck the losers who use header compression
+			memcpy(Context.Buffer, TCC->CompressedPrivateData, TCC->CompressedPrivateDataSize);
+		}
+		else if (Context.BufferSize < FrameSize) {
 			Context.BufferSize = FrameSize;
 			Context.Buffer = (uint8_t *)realloc(Context.Buffer, Context.BufferSize + 16);
 			if (Context.Buffer == NULL)
@@ -173,8 +238,13 @@ void ReadFrame(uint64_t FilePos, unsigned int &FrameSize, CompressedStream *CS,
 					"Out of memory");
 		}
 
-		size_t ReadBytes = fread(Context.Buffer, 1, FrameSize, Context.ST.fp);
+		uint8_t *TargetPtr = Context.Buffer;
+		if (TCC && TCC->CompressionMethod == COMP_PREPEND)
+			TargetPtr += TCC->CompressedPrivateDataSize;
+
+		size_t ReadBytes = fread(TargetPtr, 1, FrameSize, Context.ST.fp);
 		if (ReadBytes != FrameSize) {
+			return;
 			if (ReadBytes == 0) {
 				if (feof(Context.ST.fp)) {
 					throw FFMS_Exception(FFMS_ERROR_PARSER, FFMS_ERROR_FILE_READ,
@@ -267,6 +337,8 @@ CodecID MatroskaToFFCodecID(char *Codec, void *CodecPrivate, unsigned int FourCC
 							case 32: CID = CODEC_ID_PCM_S32BE; break;
 						}	
 						break;
+					default:
+						break;
 				}
 				
 				return CID;
@@ -353,24 +425,52 @@ void InitializeCodecContextFromHaaliInfo(CComQIPtr<IPropertyBag> pBag, AVCodecCo
 
 #endif
 
+
+// All this filename chikanery that follows is supposed to make sure both local
+// codepage (used by avisynth etc) and UTF8 (potentially used by API users) strings
+// work correctly on Win32.
+// It's a really ugly hack, and I blame Microsoft for it.
+#ifdef _WIN32
+static wchar_t *dup_char_to_wchar(const char *s, unsigned int cp) {
+	wchar_t *w;
+	int l;
+	if (!(l = MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1, NULL, 0)))
+		return NULL;
+	if (!(w = (wchar_t *)malloc(l * sizeof(wchar_t))))
+		return NULL;
+	if (MultiByteToWideChar(cp, MB_ERR_INVALID_CHARS, s, -1 , w, l) <= 0) {
+		free(w);
+		w = NULL;
+	}
+	return w;
+}
+#endif
+
 FILE *ffms_fopen(const char *filename, const char *mode) {
+#ifdef _WIN32
+	int codepage = CP_ACP;
 #ifdef FFMS_USE_UTF8_PATHS
-	// Hack: support utf8-in-char* filenames on windows
-	wchar_t filename_wide[MAX_PATH*2];
-	// 64 characters of mode string ought to be more than enough for everyone
-	wchar_t mode_wide[64];
-	if ((MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, filename, -1, filename_wide, MAX_PATH) > 0)
-			&& (MultiByteToWideChar(CP_ACP, NULL, mode, -1, mode_wide, 60) > 0))
-		return _wfopen(filename_wide, mode_wide);
+	codepage = CP_UTF8;
+#endif
+	FILE *ret;
+	wchar_t *filename_wide	= dup_char_to_wchar(filename, codepage);
+	wchar_t *mode_wide		= dup_char_to_wchar(mode, codepage);
+	if (filename_wide && mode_wide)
+		ret = _wfopen(filename_wide, mode_wide);
 	else
-		return fopen(filename, mode);
+		ret = fopen(filename, mode);
+
+	free(filename_wide);
+	free(mode_wide);
+
+	return ret;
 #else
 	return fopen(filename, mode);
-#endif
+#endif /* _WIN32 */
 }
 
 size_t ffms_mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max) {
-#ifdef FFMS_USE_UTF8_PATHS
+#if defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS)
 	// try utf8 first
 	int len = MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, mbstr, -1, NULL, 0);
 	if (len > 0) {
@@ -390,22 +490,85 @@ size_t ffms_mbstowcs (wchar_t *wcstr, const char *mbstr, size_t max) {
 
 // ffms_fstream stuff
 void ffms_fstream::open(const char *filename, std::ios_base::openmode mode) {
-#ifdef FFMS_USE_UTF8_PATHS
-	// Hack: support utf8-in-char* filenames on windows
-	wchar_t filename_wide[MAX_PATH*2];
-	if (MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, filename, -1, filename_wide, MAX_PATH) > 0)
+	// Unlike MSVC, mingw's iostream library doesn't have an fstream overload
+	// that takes a wchar_t* filename, which means you can't open unicode
+	// filenames with it on Windows. gg.
+#if defined(_WIN32) && !defined(__MINGW32__)
+	unsigned int codepage = CP_ACP;
+#if defined(FFMS_USE_UTF8_PATHS)
+	codepage = CP_UTF8;
+#endif /* defined(FFMS_USE_UTF8_PATHS) */
+	wchar_t *filename_wide = dup_char_to_wchar(filename, codepage);
+	if (filename_wide)
 		std::fstream::open(filename_wide, mode);
 	else
 		std::fstream::open(filename, mode);
-#else
+
+	free(filename_wide);
+#else /* defined(_WIN32) && !defined(__MINGW32__) */
 	std::fstream::open(filename, mode);
-#endif
+#endif /* defined(_WIN32) && !defined(__MINGW32__) */
 }
 
 ffms_fstream::ffms_fstream(const char *filename, std::ios_base::openmode mode) {
 	open(filename, mode);
 }
 
+
+#if defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS)
+int ffms_wchar_open(const char *fname, int oflags, int pmode) {
+    wchar_t *wfname = dup_char_to_wchar(fname, CP_UTF8);
+    if (wfname) {
+        int ret = _wopen(wfname, oflags, pmode);
+        av_free(wfname);
+        return ret;
+    }
+    return -1;
+}
+
+static int ffms_lavf_file_open(URLContext *h, const char *filename, int flags) {
+    int access;
+    int fd;
+
+    av_strstart(filename, "file:", &filename);
+
+    if (flags & URL_RDWR) {
+        access = _O_CREAT | _O_TRUNC | _O_RDWR;
+    } else if (flags & URL_WRONLY) {
+        access = _O_CREAT | _O_TRUNC | _O_WRONLY;
+    } else {
+        access = _O_RDONLY;
+    }
+#ifdef _O_BINARY
+    access |= _O_BINARY;
+#endif
+    fd = ffms_wchar_open(filename, access, 0666);
+    if (fd == -1)
+        return AVERROR(ENOENT);
+    h->priv_data = (void *) (intptr_t) fd;
+    return 0;
+}
+
+// Hijack lavf's file protocol handler's open function and use our own instead.
+// Hack by nielsm.
+void ffms_patch_lavf_file_open() {
+	extern URLProtocol *first_protocol;
+	URLProtocol *proto = first_protocol;
+	while (proto != NULL) {
+		if (strcmp("file", proto->name) == 0) {
+			break;
+		}
+		proto = proto->next;
+	}
+	if (proto != NULL) {
+		proto->url_open = &ffms_lavf_file_open;
+	}
+}
+#endif /* defined(_WIN32) && defined(FFMS_USE_UTF8_PATHS) */
+
+// End of filename hackery.
+
+
 #ifdef HAALISOURCE
 
 CComPtr<IMMContainer> HaaliOpenFile(const char *SourceFile, enum FFMS_Sources SourceMode) {
@@ -490,3 +653,13 @@ void CorrectNTSCRationalFramerate(int *Num, int *Den) {
 		}
 	}
 }
+
+// correct the timebase if it is invalid
+void CorrectTimebase(FFMS_VideoProperties *VP, FFMS_TrackTimeBase *TTimebase) {
+	double Timebase = (double)TTimebase->Num / TTimebase->Den;
+	double FPS = (double)VP->FPSNumerator / VP->FPSDenominator;
+	if ((1000/Timebase) / FPS < 1) {
+		TTimebase->Den = VP->FPSNumerator;
+		TTimebase->Num = (int64_t)VP->FPSDenominator * 1000;
+	}
+}
